﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Shell;

namespace PhoneAppLLS
{
    public partial class EditEntry : PhoneApplicationPage
    {

        // these are the toolbar buttons
        private enum AppButtons
        {
            ButOK,
            ButDelete
        };

        /// <summary>
        /// When true the user has pressed OK and the entry has been accepted
        /// </summary>
        public bool OK
        {
            set;
            get;
        }

        /// <summary>
        /// When true the item is being edited. This is a state that is set when the form
        /// is invoked - so it changes the title and the way the entry is validated when the
        /// user presses OK
        /// </summary>
        private bool IsEdit
        {
            set;
            get;
        }

        /// <summary>
        /// This is the information to be edited and will be the value
        /// passed back to the caller
        /// </summary>
        public VaultEntry VaultEntry
        {
            set;
            get;
        }

        public EditEntry()
        {
            InitializeComponent();

            OK = false;
            IsEdit = false;
            VaultEntry = new VaultEntry();
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            // if the password has not been entered then go back to prior screen
            if ((Application.Current as App).DAL.ResumeFromDeactivation)
            {
                // step back to the previous screen
                this.NavigationService.GoBack();
                return;
            }

            if (args.Content is EditEntry)
            {
                IDictionary<string, string> parameters = this.NavigationContext.QueryString;
                if (parameters.ContainsKey("type"))
                {
                    // set the state to editing
                    IsEdit = parameters["type"] == "edit";
                }

                VaultEntry = new VaultEntry();

                PageTitle.Text = (IsEdit) ? "edit entry" : "new entry";

                DataAccessLayer DAL = (Application.Current as App).DAL;

                if (IsEdit)
                {
                    // invoked via an index into the ValueEntry dictionary
                    if (parameters.ContainsKey("index"))
                    {
                        int index = Convert.ToInt16(parameters["index"]);
                        VaultEntry = DAL.Get(index);
                    }

                    // invoked via the key
                    if (parameters.ContainsKey("key"))
                    {
                        string key = Convert.ToString(parameters["key"]);
                        VaultEntry = DAL.Get(key);
                    }
                }

                textBoxCaption.Text = VaultEntry.Caption;
                textBoxIdentity.Text = VaultEntry.Identity;
                textBoxPassword.Text = VaultEntry.Password;
                textBoxNote.Text = VaultEntry.OtherInfo;
            }

            base.OnNavigatedTo(args);
        }

        /// <summary>
        /// This is called just prior to leaving the page and give access to the callers context
        /// </summary>
        /// <param name="args"></param>
        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            // if we are going back to EditVault page then set the value
            if (args.Content is EditEntry)
            {
                if (OK)
                {
                    //(args.Content as EditVault).CurrentSelection = this.VaultEntry;
                }
            }

            base.OnNavigatedFrom(args);
        }

        /// <summary>
        /// original = ABC
        /// 
        /// if new=ABC && editing then delete ABC, add ABC
        /// if new!=ABC but ABC exists && editing then 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOK_Click(object sender, EventArgs e)
        {
            try
            {
                DataAccessLayer DAL = (Application.Current as App).DAL;

                // capture the information - trim the caption as white space is confusing
                string caption = textBoxCaption.Text.Trim();

                // if editing then replace the existing entry
                if (IsEdit)
                {
                    // if the caption has not changed then it will be managed by the Add process
                    // so these tests only need to be undertaken when the user has changed the caption in some way
                    if (VaultEntry.Caption != caption)
                    {
                        // if the caption matches another existing entry then exit
                        if (DAL.Get(caption) != null)
                        {
                            MessageBox.Show("This entry already exists, please either change the caption or edit the original entry");
                            return;
                        }
                        else
                        {
                            // if the caption has changed then delete the existing entry
                            if (VaultEntry.Caption != caption)
                            {
                                if ((DAL.Get(VaultEntry.Caption) != null))
                                {
                                    DAL.Delete(VaultEntry.Caption);
                                }
                            }
                        }
                    }
                }
                else
                {
                    // new entry - so if the caption already exists then exit with error
                    if (DAL.Get(caption) != null)
                    {
                        MessageBox.Show("This entry already exists, please either change the caption or edit the original entry");
                        return;
                    }
                }

                // put the entry into place
                VaultEntry.Caption = caption;
                VaultEntry.Identity = textBoxIdentity.Text;
                VaultEntry.Password = textBoxPassword.Text;
                VaultEntry.OtherInfo = textBoxNote.Text;

                // put it back into the vault
                DAL.Add(VaultEntry);

                // ok entry is valid so exit
                OK = true;

                // and then return to the caller
                NavigationService.GoBack();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "error", MessageBoxButton.OK);
            }
        }

        private void textBoxCaption_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (VaultEntry != null)
            {
                DataAccessLayer DAL = (Application.Current as App).DAL;

                // can only delete if the caption has not been changed
                bool state = (VaultEntry.Caption == textBoxCaption.Text);

                // can only delete if in edit mode? 
                // can only delete if there is more than 1 entry
                if (IsEdit == false || DAL.VaultEntries.Count == 1) state = false;

                (ApplicationBar.Buttons[(int)AppButtons.ButDelete] as ApplicationBarIconButton).IsEnabled = state;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            // ok any entry changes are being cancelled so exit
            OK = false;

            // and then return to the caller
            NavigationService.GoBack();
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            try
            {
                DataAccessLayer DAL = (Application.Current as App).DAL;

                // do not allow delete when there is only a single entry
                if (DAL.VaultEntries.Count() == 1)
                {
                    throw new Exception("there has to be at least one entry in the vault");
                }

                string key = textBoxCaption.Text;

                // this should only be enabled when an item has been selected - maybe with a confirm?
                MessageBoxResult result = MessageBox.Show(string.Format("are you sure you want to delete: {0}?", key), "delete", MessageBoxButton.OKCancel);
                if (result == MessageBoxResult.OK)
                {
                    // delete from the Vault
                    DAL.Delete(key);

                    // ok entry is valid so exit
                    OK = false;

                    // and then return to the caller
                    NavigationService.GoBack();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "error", MessageBoxButton.OK);
            }
        }
    }
}