﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;
using System.Xml.Linq;
using XElementHelpers;

namespace PhoneAppLLS
{
    public class VaultEntry
    {
        /// <summary>
        /// The visible entry name
        /// </summary>
        public string Caption
        {
            set;
            get;
        }

        /// <summary>
        /// User name or other identity such as email
        /// </summary>
        public string Identity
        {
            set;
            get;
        }

        /// <summary>
        /// The password - it will be encrypted using the system password
        /// </summary>
        public string Password
        {
            set;
            get;
        }

        /// <summary>
        /// Addtional text that may be provided for this entry
        /// </summary>
        public string OtherInfo
        {
            set;
            get;
        }

        /// <summary>
        /// Default constructor
        /// </summary>
        public VaultEntry()
        {
            this.Caption = "";
            this.Identity = "";
            this.Password = "";
            this.OtherInfo = "";
        }

        /// <summary>
        /// Constructor with full set of initial parameters
        /// </summary>
        /// <param name="Caption"></param>
        /// <param name="Password"></param>
        public VaultEntry(string Caption, string Identity, string Password, string OtherInfo)
        {
            this.Caption = Caption;
            this.Identity = Identity;
            this.Password = Password;
            this.OtherInfo = OtherInfo;
        }

        /// <summary>
        /// Load the indexed entry from storage
        /// </summary>
        /// <param name="Index"></param>
        public VaultEntry(int Index)
        {
            IsolatedStorageSettings settings = IsolatedStorageSettings.ApplicationSettings;

            string val = "";

            // pick out the entry from storage
            settings.TryGetValue(string.Format("Entry_{0}", Index), out val);
            if (val != null)
            {
                XElement xEntry = XElement.Parse(val);

                Caption = xEntry.Element("Caption").ToStringNull();
                Identity = xEntry.Element("Identity").ToStringNull();
                Password = xEntry.Element("Password").ToStringNull();
                OtherInfo = xEntry.Element("OtherInfo").ToStringNull();
            }
        }

        /// <summary>
        /// Create and return a clone of the current entry
        /// </summary>
        /// <returns></returns>
        public VaultEntry Clone()
        {
            VaultEntry entry = new VaultEntry();
            entry.Caption = this.Caption;
            entry.Identity = this.Identity;
            entry.Password = this.Password;
            entry.OtherInfo = this.OtherInfo;

            return entry;
        }

        /// <summary>
        /// Return the current entry as XML
        /// </summary>
        /// <param name="Index"></param>
        /// <returns></returns>
        public XElement SaveEntryXML()
        {
            XElement xEntry = new XElement("Entry",
                new XElement("Caption", Caption),
                new XElement("Identity", Identity),
                new XElement("Password", Password),
                new XElement("OtherInfo", OtherInfo));

            return xEntry;
        }

        /// <summary>
        /// Load the entry from the XML
        /// </summary>
        /// <param name="VaultEntry"></param>
        public void LoadEntryXML(XElement VaultEntry)
        {
            Caption = VaultEntry.Element("Caption").ToStringNull();
            Identity = VaultEntry.Element("Identity").ToStringNull();
            Password = VaultEntry.Element("Password").ToStringNull();
            OtherInfo = VaultEntry.Element("OtherInfo").ToStringNull();
        }

        /// <summary>
        /// Save the indexed entry to storage as XML
        /// </summary>
        /// <param name="Index"></param>
        public void SaveEntry(int Index)
        {
            IsolatedStorageSettings settings = IsolatedStorageSettings.ApplicationSettings;

            settings[string.Format("Entry_{0}", Index)] = SaveEntryXML().ToString();
        }

        /// <summary>
        /// Get the first letter of the caption or #
        /// </summary>
        public static string GetCaptionGroup(VaultEntry Current)
        {
            char key = char.ToLower(Current.Caption[0]);

            if (key < 'a' || key > 'z')
            {
                key = '#';
            }

            return key.ToString();
        }

        /// <summary>
        /// Sort by caption
        /// </summary>
        /// <param name="obj1"></param>
        /// <param name="obj2"></param>
        /// <returns></returns>
        public static int CompareCaptions(object obj1, object obj2)
        {
            VaultEntry p1 = (VaultEntry)obj1;
            VaultEntry p2 = (VaultEntry)obj2;

            int result = p1.Caption.CompareTo(p2.Caption);
            if (result == 0)
            {
                result = p1.Caption.CompareTo(p2.Caption);
            }

            return result;
        }
    }
}
