﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;

namespace XElementHelpers
{
    public static class XElementHelpers
    {
        /// <summary>
        /// Convert the element value to an int - returning 0 if the element is missing, null or empty
        /// 
        /// An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static int ToInt0(this XElement dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return 0;
            }
            else
            {
                return Convert.ToInt32(dt.Value);
            }
        }

        /// <summary>
        /// Convert the attribute value to an int - returning 0 if the element is missing, null or empty
        /// 
        /// An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static int ToInt0(this XAttribute dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return 0;
            }
            else
            {
                return Convert.ToInt32(dt.Value);
            }
        }

        /// <summary>
        /// Convert the element value to a double - returning 0 if the element is missing, null or empty
        /// 
        /// An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static double ToDouble0(this XElement dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return 0;
            }
            else
            {
                return Convert.ToDouble(dt.Value);
            }
        }

        /// <summary>
        /// Convert the element value to a bool - returning false if the element is missing, null or empty
        /// 
        /// An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static bool ToBool(this XElement dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return false;
            }
            else
            {
                return Convert.ToBoolean(dt.Value);
            }
        }

        /// <summary>
        /// Convert the attribute value to a bool - returning false if the element is missing, null or empty
        /// 
        /// An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static bool ToBool(this XAttribute dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return false;
            }
            else
            {
                return Convert.ToBoolean(dt.Value);
            }
        }

        /// <summary>
        /// Convert the element value to a datetime - returning first day of the century if the element is missing, null or empty
        /// 
        ///  An exception will be thrown if the conversion fails
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static DateTime ToDate(this XElement dt)
        {
            if (dt == null || dt.Value == null || dt.Value == "")
            {
                return Convert.ToDateTime("1 Jan 2000");
            }
            else
            {
                return Convert.ToDateTime(dt.Value);
            }
        }

        /// <summary>
        /// Convert the element value to a string - returning empty string if the element is missing
        /// 
        /// If the element is missing an empty string is returned
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static string ToStringNull(this XElement dt)
        {
            if (dt == null)
            {
                return "";
            }
            else
            {
                return dt.Value;
            }
        }

        /// <summary>
        /// Convert the attribute value to a string - returning empty string if the element is missing
        /// 
        /// If the element is missing an empty string is returned
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static string ToStringNull(this XAttribute dt)
        {
            if (dt == null)
            {
                return "";
            }
            else
            {
                return dt.Value;
            }
        }

        ///// <summary>
        ///// Convert an XMLNode into an XElement
        ///// </summary>
        ///// <param name="node"></param>
        ///// <returns></returns>
        //public static XElement GetXElement(this XmlNode node)
        //{
        //    XDocument xDoc = new XDocument();

        //    using (XmlWriter xmlWriter = xDoc.CreateWriter())
        //        node.WriteTo(xmlWriter);

        //    return xDoc.Root;
        //}

        ///// <summary>
        ///// Convert an XElement into an XmlElement
        ///// </summary>
        ///// <param name="element"></param>
        ///// <returns></returns>
        //public static XmlElement GetXmlElement(this XElement element)
        //{
        //    using (XmlReader xmlReader = element.CreateReader())
        //    {
        //        XmlDocument xmlDoc = new XmlDocument();

        //        xmlDoc.Load(xmlReader);

        //        return xmlDoc.DocumentElement;
        //    }
        //}
    }
}